# frozen_string_literal: true

# Heart of the application. It renders HTML to PDF.
class RenderGrover < ActiveInteraction::Base
  interface :params, methods: %i[[] each]

  # ################################################
  # Validation
  # ################################################
  validate :params_url_or_html_presence
  validate :output_format_validity

  attr_reader :headers

  def execute
    initialize_headers

    @grover = Grover.new(grover_html_or_url, **grover_options)

    process_generation
  end

  def process_generation
    case output_format.to_s
    when 'pdf'
      @headers['Content-Type'] = 'application/pdf'
      @grover.to_pdf
    when 'png'
      @headers['Content-Type'] = 'image/png'
      @grover.to_png
    when 'jpeg'
      @headers['Content-Type'] = 'image/jpeg'
      @grover.to_jpeg
    when 'html'
      @headers['Content-Type'] = 'text/html'
      @grover.to_html
    else
      raise 'Invalid output format'
    end
  end

  def initialize_headers
    @headers = {
      'Content-Disposition' => "#{content_disposition_location}; filename=\"#{params[:filename] || 'file'}.#{output_format}\"",
      "Access-Control-Expose-Headers" => "*",
      "Cache-Control" => "no-cache, no-store, must-revalidate",
      "Pragma" => "no-cache",
      "Expires" => "0",
      "X-Grover-Version" => Grover::VERSION,
      "X-Grover-Options" => grover_options.to_json,
      "X-Request-Params" => params.to_json,
      "X-Request-Output-Format" => output_format,
      "X-Grover-Url-Or-Html" => grover_html_or_url,
    }
  end

  def grover_html_or_url
    params[:url] || params[:html]
  end

  def content_disposition_location
    @content_disposition_location ||= %w[inline attachment].include?(params[:content_disposition_location].to_s.downcase.strip) ? params[:content_disposition_location].to_s.downcase.strip : 'attachment'
  end

  def output_format
    @output_format ||= params[:output_format] || 'pdf'
  end

  def grover_options
    return @grover_options if defined?(@grover_options)

    @grover_options = params[:grover_options] || {}
    @grover_options = JSON.parse(@grover_options) if @grover_options.is_a?(String) && @grover_options.valid_json?

    %w[
      format
      margin
      user_agent
      viewport
      prefer_css_page_size
      emulate_media
      bypass_csp
      media_features
      timezone
      vision_deficiency
      extra_http_headers
      geolocation
      focus
      hover
      cache
      timeout
      request_timeout
      convert_timeout
      launch_args
      wait_until
    ].each do |key|
      @grover_options[key] = params["grover_#{key}"] if params["grover_#{key}"].present?
    end

    @grover_options
  end

  def params_url_or_html_presence
    return if params[:html].present? || params[:url].present?

    errors.add(:params, 'html or url must be present')
  end

  def output_format_validity
    return if %w[pdf png jpeg html].include?(output_format.to_s)

    errors.add(:output_format, 'Invalid output format')
  end
end