# frozen_string_literal: true

# Heart of the application. It renders HTML to PDF.
class RenderHtmlToPdf < ActiveInteraction::Base
  # ################################################
  # Class methods
  # ################################################
  class << self
    def filters_description
      params_description
    end

    # { key: 'description', key2: 'description2' }
    def params_description
      filters.keys.map { |filter_key| [filter_key, filters[filter_key].desc || ''] }.to_h.with_indifferent_access
    end
  end

  # ################################################
  # Inputs
  # Adding inputs here will reflect in all the application.
  # This inputs will be allowed to be passed to the interaction, and to the /render/ endpoint.
  # The description will be used to generate the documentation, which will be public.
  # ################################################
  string :html, default: nil, desc: 'HTML to render'
  string :url, default: nil, desc: 'URL where get HTML to render from'
  interface :grover_options, default: {}, desc: 'Options to pass to Grover. Object {}. See https://github.com/Studiosity/grover.', methods: %i[each merge keys]
  string :filename, default: 'file', desc: 'Filename to use when downloading the PDF. Without extension.'
  interface :url_additional_params, methods: %i[to_query], default: nil

  # ################################################
  # Entry point
  # ################################################
  def execute
    validate_inputs
    # return errors unless valid?

    initialize_grover
    # grover_pdf_file
  end

  def filename_with_extension
    "#{filename_without_extension || 'file'}.pdf"
  end

  # ################################################
  # Private inputs
  # ################################################
  private

  def filename_without_extension
    filename.index('.pdf') ? filename.gsub('.pdf', '') : filename
  end

  def validate_inputs
    validate_html_or_url_presence
  end

  def validate_html_or_url_presence
    return unless html.blank? && url.blank?

    errors.add(:base, 'html or url must be present')
    errors.add(:html, 'html or url must be present')
    errors.add(:url, 'html or url must be present')
  end

  def initialize_grover
    @grover = Grover.new(html || url_with_additional_params, **(_grover_options.to_h.transform_keys(&:to_sym)))
  end

  def url_with_additional_params
    return url unless url_additional_params

    return "#{url}?#{url_additional_params.to_query}" unless url.include?('?')

    "#{url}&#{url_additional_params.to_query}"
  end

  def grover_pdf_source
    @grover.to_pdf
  end

  # Merging default grover options with the ones passed as input
  def _grover_options
    # DEFAULT_GROVER_OPTIONS.with_indifferent_access.merge(grover_options.with_indifferent_access)
    grover_options.with_indifferent_access || {}
  end
end
